package com.persagy.account.controller;

import java.util.List;
import java.util.Map;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.google.common.collect.Lists;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;
import com.persagy.account.manage.SaasAuthHandler;
import com.persagy.account.manage.SaasGroupProjectHandler;
import com.persagy.account.mq.CommonTopicProducer;
import com.persagy.account.pojo.dto.SaasGroup;
import com.persagy.account.pojo.vo.group.SaasGroupCreateVO;
import com.persagy.account.pojo.vo.group.SaasGroupQueryVO;
import com.persagy.account.service.ISaasGroupService;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 集团信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-03-13 15:29:50
 */
@Api(tags = "集团信息")
@RestController
@RequestMapping(value = "/group", method = RequestMethod.POST)
public class SaasGroupController {
	
	@Autowired
	private SaasAuthHandler saasAuthHandler;
	
    @Autowired
    private ISaasGroupService saasGroupService;
    
	@Autowired
	private CommonTopicProducer commonTopicProducer;
    
    @Autowired
    private SaasGroupProjectHandler saasGroupProjectHandler;
    
    /**
     * 列表查询
     */
    @ApiOperation(value = "列表查询")
    @RequestMapping(value = "querySaasGroupList")
    public ResponseResult querySaasGroupList(@RequestBody @Validated SaasGroupQueryVO queryVO) {
    	// 账号数据可见性的权限鉴定
		Map<String, String> authMap = this.saasAuthHandler.validAccountAuth(queryVO.getAccountId(), queryVO.getGroupCode());
		if (StringUtil.isNotBlank(authMap.get(SaasCommonConstant.GROUP_CODE))) {
			queryVO.setGroupCode(authMap.get(SaasCommonConstant.GROUP_CODE));
		}
    	// 添加数据访问控制，账号可见域，0-所有集团项目，1-单集团所有项目(此时，集团编码需存在)，2-其他
		String accountType = authMap.get(SaasCommonConstant.ACCOUNT_TYPE);
    	// 2-其他
		if (SaasCommonConstant.STR_STATUS_2.equals(accountType)) {
			SaasGroup saasGroup = new SaasGroup();
			BeanUtils.copyProperties(queryVO, saasGroup);
			List<SaasGroup> list = this.saasGroupService.queryAllowGroupInfo(saasGroup, queryVO.getAccountId());
			return ResponseResultUtil.successResult(list, CollectionUtil.isEmpty(list) ? 0L : list.size());
		}
		List<SaasGroup> list = this.saasGroupProjectHandler.querySaasGroupList(queryVO);
		if (CollectionUtil.isEmpty(list)) {
			return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
        return ResponseResultUtil.successResult(list, (long)list.size());
    }
    
    /**
     * 查询集团项目树，条件只生效账号ID和集团编码字段
     */
    @ApiOperation(value = "查询集团项目树")
    @RequestMapping(value = "queryGroupProjectTree")
    public ResponseResult queryGroupProjectTree(@RequestBody @Validated SaasGroupQueryVO queryVO) {
    	// 账号数据可见性的权限鉴定
		Map<String, String> authMap = this.saasAuthHandler.validAccountAuth(queryVO.getAccountId(), queryVO.getGroupCode());
		if (StringUtil.isNotBlank(authMap.get(SaasCommonConstant.GROUP_CODE))) {
			queryVO.setGroupCode(authMap.get(SaasCommonConstant.GROUP_CODE));
		}
    	// 添加数据访问控制，账号可见域，0-所有集团项目，1-单集团所有项目(此时，集团编码需存在)，2-其他
		String accountType = authMap.get(SaasCommonConstant.ACCOUNT_TYPE);
    	// 2-其他
		if (SaasCommonConstant.STR_STATUS_2.equals(accountType)) {
			List<SaasGroup> list = this.saasGroupProjectHandler.queryGroupProjectTree(queryVO.getAccountId(), queryVO.getGroupCode(), false);
	        return ResponseResultUtil.successResult(list, (long)list.size());
		}
		List<SaasGroup> list = this.saasGroupProjectHandler.queryGroupProjectTree(queryVO.getAccountId(), queryVO.getGroupCode(), true);
        return ResponseResultUtil.successResult(list, (long)list.size());
    }
    
    /**
     * 新增，同时新增权限记录，也同时新增一条未分区的区域信息
     */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasGroup")
    public ResponseResult createSaasGroup(@RequestBody @Validated SaasGroupCreateVO createVO) {
		// 验证集团名称与集团编码的分别唯一性
		boolean exists = this.saasGroupService.validGroupCodeOrName(createVO.getGroupCode(), createVO.getGroupName(), false);
		if (!exists) {
			return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "集团编码或集团名称已存在");
		}
		SaasGroup saasGroup = new SaasGroup();
		BeanUtils.copyProperties(createVO, saasGroup);
		saasGroup.setUpdateUser(createVO.getAccountId());
        return this.saasGroupProjectHandler.createSaasGroup(saasGroup, createVO.getAccountId());
    }
    
    /**
     * 更新，无需新增权限记录
     */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasGroup")
    public ResponseResult updateSaasGroup(@RequestBody @Validated SaasGroupCreateVO createVO) {
		// 验证数据是否名称是否重复
		boolean exists = this.saasGroupService.validGroupCodeOrName(createVO.getGroupCode(), createVO.getGroupName(), true);
		if (!exists) {
			return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "集团名称，已存在");
		}
		SaasGroup saasGroup = new SaasGroup();
		BeanUtils.copyProperties(createVO, saasGroup);
		saasGroup.setUpdateUser(createVO.getAccountId());
		LambdaUpdateWrapper<SaasGroup> updateWrapper = new SaasGroup.Builder().createUpdateWrapper()
				.groupCodeEq(createVO.getGroupCode()).validEq(SaasCommonConstant.STATUS_1).builderUpdateWrapper();
		exists = this.saasGroupService.update(saasGroup, updateWrapper);
		if (exists) {
			this.commonTopicProducer.sendGroupInfo(saasGroup, SaasCommonConstant.UPDATE_TYPE);
			return ResponseResultUtil.successResult("更新成功");
		}
        return ResponseResultUtil.errorResult("更新失败");
    }

    /**
     * 逻辑删除
     */
	/*@ApiOperation(value = "删除")
    @RequestMapping(value = "deleteSaasGroup")
    public ResponseResult deleteSaasGroup(@RequestBody SaasGroup saasGroup) {
        boolean result = this.saasGroupService.removeById(saasGroup.getId());
        return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
    }*/
    
}
