package com.persagy.account.controller;

import java.util.List;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.persagy.account.pojo.dto.SaasGroupProduct;
import com.persagy.account.pojo.dto.SaasProjectProduct;
import com.persagy.account.pojo.vo.product.SaasGroupProductCreateVO;
import com.persagy.account.pojo.vo.product.SaasGroupProductDeleteVO;
import com.persagy.account.pojo.vo.product.SaasGroupProductPageVO;
import com.persagy.account.pojo.vo.product.SaasGroupProductQueryVO;
import com.persagy.account.service.ISaasGroupProductService;
import com.persagy.account.service.ISaasProjectProductService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 集团签约产品
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-05-07 12:02:40
 */
@RestController
@SuppressWarnings("unchecked")
@Api(tags = "产品集团签约管理")
@RequestMapping(value = "/groupProduct", method = RequestMethod.POST)
public class SaasGroupProductController {
	
    @Autowired
    private ISaasGroupProductService saasGroupProductService;
    
    @Autowired
    private ISaasProjectProductService saasProjectProductService;
    
    /**
     * 查询产品集团签约信息
     */
    @ApiOperation(value = "普通列表查询")
    @RequestMapping(value = "querySaasGroupProductList")
    public ResponseResult querySaasGroupProductList(@RequestBody @Validated SaasGroupProductQueryVO model) {
    	LambdaQueryWrapper<SaasGroupProduct> queryWrapper = new SaasGroupProduct.BuilderQueryWrapper()
    			.groupCodeEq(model.getGroupCode()).productIdEq(model.getProductId()).validLastEq(model.getValidLast()).builder();
    	List<SaasGroupProduct> list = this.saasGroupProductService.list(queryWrapper);
    	if (CollectionUtil.isEmpty(list)) {
			return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
        return ResponseResultUtil.successResult(list, (long)list.size());
    }
    
    /**
     * 分页查询产品集团签约信息
     */
	@ApiOperation(value = "分页列表查询")
    @RequestMapping(value = "queryGroupProductPageList")
    public ResponseResult queryGroupProductPageList(@RequestBody @Validated SaasGroupProductPageVO model) {
		Page<SaasGroupProduct> page = new Page<SaasGroupProduct>(model.getPage(), model.getSize());
		LambdaQueryWrapper<SaasGroupProduct> queryWrapper = new SaasGroupProduct.BuilderQueryWrapper()
    			.groupCodeEq(model.getGroupCode()).productIdEq(model.getProductId()).validLastEq(model.getValidLast()).builder();
		queryWrapper.orderByDesc(SaasGroupProduct::getProductContract, SaasGroupProduct::getId);		// 默认签约时间降序
		Page<SaasGroupProduct> pageList = this.saasGroupProductService.page(page, queryWrapper);
    	if (pageList == null || CollectionUtil.isEmpty(pageList.getRecords())) {
    		return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
    	List<SaasGroupProduct> records = page.getRecords();
        if (model.isTotalProject()) {
			for (SaasGroupProduct saasGroupProduct : records) {
				LambdaQueryWrapper<SaasProjectProduct> projectWrapper = new SaasProjectProduct.BuilderQueryWrapper().groupCodeEq(saasGroupProduct.getGroupCode()).productIdEq(saasGroupProduct.getProductId()).builder();
				saasGroupProduct.setProjectNum(this.saasProjectProductService.count(projectWrapper));
			}
		}
    	return ResponseResultUtil.successResult(records, pageList.getTotal());
    }

    /**
     * 新增产品集团签约信息
     */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasGroupProduct")
    public ResponseResult createSaasGroupProduct(@RequestBody @Validated SaasGroupProductCreateVO model) {
		if (SaasCommonConstant.STR_STATUS_0.equals(model.getValidLast())) {
			if (!(model.getValidStartTime() != null && model.getValidEndTime() != null)) {
				return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "当validLast为0时，签约有效期时间字段不可为空");
			}
		}
		SaasGroupProduct saasGroupProduct = new SaasGroupProduct();
		BeanUtils.copyProperties(model, saasGroupProduct);
		saasGroupProduct.setId(null);
		saasGroupProduct.setUpdateUser(model.getAccountId());
		boolean result = this.saasGroupProductService.createSaasGroupProduct(saasGroupProduct, model.getPageIds());
        return result ? ResponseResultUtil.successResult("保存成功", saasGroupProduct.getId()) : ResponseResultUtil.errorResult("保存失败");
    }
    
    /**
     * 更新产品集团签约信息
     */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasGroupProduct")
    public ResponseResult updateSaasGroupProduct(@RequestBody @Validated SaasGroupProductCreateVO model) {
		if (StringUtil.isBlank(model.getId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "签约ID不可为空");
		}
		if (SaasCommonConstant.STR_STATUS_0.equals(model.getValidLast())) {
			if (!(model.getValidStartTime() != null && model.getValidEndTime() != null)) {
				return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "当validLast为0时，签约有效期时间字段不可为空");
			}
		}
		SaasGroupProduct saasGroupProduct = new SaasGroupProduct();
		BeanUtils.copyProperties(model, saasGroupProduct);
		saasGroupProduct.setUpdateUser(model.getAccountId());
        boolean result = this.saasGroupProductService.updateSaasGroupProduct(saasGroupProduct, model.getPageIds());
        return result ? ResponseResultUtil.successResult("更新成功") : ResponseResultUtil.errorResult("更新失败");
    }

    /**
     * 物理删除产品集团签约信息
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "deleteSaasGroupProduct")
    public ResponseResult deleteSaasGroupProduct(@RequestBody @Validated SaasGroupProductDeleteVO model) {
    	// 获取签约ID
    	LambdaQueryWrapper<SaasGroupProduct> queryWrapper = new SaasGroupProduct.BuilderQueryWrapper()
    			.groupCodeEq(model.getGroupCode()).productIdEq(model.getProductId()).builder();
    	SaasGroupProduct saasGroupProduct = this.saasGroupProductService.getOne(queryWrapper);
    	if (saasGroupProduct == null) {
    		return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "该集团未签约此产品");
		}
        boolean result = this.saasGroupProductService.deleteSaasGroupProduct(saasGroupProduct.getId());
        return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
    }
    
}
