package com.persagy.account.controller;

import java.util.List;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.persagy.account.pojo.dto.SaasProduct;
import com.persagy.account.pojo.vo.product.SaasProductCreateVO;
import com.persagy.account.pojo.vo.product.SaasProductPageVO;
import com.persagy.account.pojo.vo.product.SaasProductQueryVO;
import com.persagy.account.service.ISaasProductService;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;

import cn.hutool.core.collection.CollectionUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 产品信息
 * 
 * @version 1.0.0
 * @company persagy 
 * @author zhangqiankun
 * @date 2021-04-27 16:08:25
 */
@RestController
@Api(tags = "产品信息")
@SuppressWarnings("unchecked")
@RequestMapping(value = "/product", method = RequestMethod.POST)
public class SaasProductController {
	
    @Autowired
    private ISaasProductService saasProductService;
    
    /**
     * 查询产品信息
     */
	@ApiOperation(value = "查询产品信息")
    @RequestMapping(value = "querySaasProductList")
    public ResponseResult querySaasProductList(@RequestBody @Validated SaasProductQueryVO model) {
    	LambdaQueryWrapper<SaasProduct> queryWrapper = new SaasProduct.BuilderQueryWrapper()
    			.productCodeEq(model.getProductCode()).productNameLike(model.getProductName())
    			.productStateEq(model.getProductState()).productMenuEq(model.getProductMenu())
    			.productTypeEq(model.getProductType()).appIdEq(model.getAppId()).updateUserEq(model.getUpdateUser()).builder();
    	queryWrapper.orderByDesc(SaasProduct::getUpdateTime, SaasProduct::getId);		// 默认更新时间降序
    	List<SaasProduct> list = this.saasProductService.list(queryWrapper);
    	if (CollectionUtil.isEmpty(list)) {
			return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
        return ResponseResultUtil.successResult(list, (long)list.size());
    }
	
    /**
     * 分页查询产品信息
     */
    @ApiOperation(value = "分页查询产品信息")
    @RequestMapping(value = "queryProductPageList")
    public ResponseResult queryProductPageList(@RequestBody @Validated SaasProductPageVO pageVO) {
    	Page<SaasProduct> page = new Page<SaasProduct>(pageVO.getPage(), pageVO.getSize());
    	LambdaQueryWrapper<SaasProduct> queryWrapper = new SaasProduct.BuilderQueryWrapper()
    			.productCodeEq(pageVO.getProductCode()).productNameLike(pageVO.getProductName())
    			.productStateEq(pageVO.getProductState()).productMenuEq(pageVO.getProductMenu())
    			.productTypeEq(pageVO.getProductType()).appIdEq(pageVO.getAppId()).updateUserEq(pageVO.getUpdateUser()).builder();
    	queryWrapper.orderByDesc(SaasProduct::getUpdateTime, SaasProduct::getId);		// 默认更新时间降序
    	Page<SaasProduct> pageList = this.saasProductService.page(page, queryWrapper);
    	if (pageList == null || CollectionUtil.isEmpty(pageList.getRecords())) {
    		return ResponseResultUtil.successResult(Lists.newArrayList(), 0L);
		}
    	return ResponseResultUtil.successResult(pageList.getRecords(), pageList.getTotal());
    }

    /**
     * 新增
     */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasProduct")
    public ResponseResult createSaasProduct(@RequestBody @Validated SaasProductCreateVO model) {
		if (StringUtil.isBlank(model.getAppId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "应用ID，不可为空"); 
		}
		// 验证产品名称、产品编码的唯一性
		boolean result = this.saasProductService.validProductCode(model.getProductCode(), null);
		if (!result) {
			return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "产品编码已存在");
		}
		result = this.saasProductService.validProductName(model.getProductName(), null);
		if (!result) {
			return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "产品名称已存在");
		}
		SaasProduct saasProduct = new SaasProduct();
		BeanUtils.copyProperties(model, saasProduct);
		saasProduct.setUpdateUser(model.getAccountId());
		saasProduct.setId(null);
        result = this.saasProductService.save(saasProduct);
        return result ? ResponseResultUtil.successResult("保存成功", saasProduct.getId()) : ResponseResultUtil.errorResult("保存失败");
    }
    
    /**
     * 更新
     */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasProduct")
    public ResponseResult updateSaasProduct(@RequestBody @Validated SaasProductCreateVO model) {
		if (StringUtil.isBlank(model.getId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "产品ID，不可为空"); 
		}
		if (StringUtil.isBlank(model.getAppId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "应用ID，不可为空"); 
		}
		// 验证产品名称、产品编码的唯一性
		boolean result = this.saasProductService.validProductCode(model.getProductCode(), model.getId());
		if (!result) {
			return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "产品编码已存在");
		}
		result = this.saasProductService.validProductName(model.getProductName(), model.getId());
		if (!result) {
			return ResponseResultUtil.errorResult(ResponseCode.C0341.getCode(), "产品名称已存在");
		}
		SaasProduct saasProduct = new SaasProduct();
		BeanUtils.copyProperties(model, saasProduct);
		saasProduct.setUpdateUser(model.getAccountId());
        result = this.saasProductService.updateById(saasProduct);
        return result ? ResponseResultUtil.successResult("更新成功") : ResponseResultUtil.errorResult("更新失败");
    }

}
