package com.persagy.account.controller;

import java.util.List;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.persagy.account.manage.SaasProductHandler;
import com.persagy.account.pojo.dto.SaasProduct;
import com.persagy.account.pojo.dto.SaasProductPage;
import com.persagy.account.pojo.vo.product.SaasProductPageCreateVO;
import com.persagy.account.pojo.vo.product.SaasProductPageDeleteVO;
import com.persagy.account.pojo.vo.product.SaasProductPageTreeVO;
import com.persagy.account.service.ISaasProductPageService;
import com.persagy.account.service.ISaasProductService;
import com.persagy.common.constant.SaasCommonConstant;
import com.persagy.common.enums.ResponseCode;
import com.persagy.common.utils.ResponseResult;
import com.persagy.common.utils.ResponseResultUtil;
import com.persagy.common.utils.StringUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 产品页面配置
 * 
 * @version 1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2021-05-06 10:12:47
 */
@RestController
@Api(tags = "产品页面配置")
@RequestMapping(value = "/productPage", method = RequestMethod.POST)
public class SaasProductPageController {

	@Autowired
	private SaasProductHandler saasProductHandler;

    @Autowired
    private ISaasProductService saasProductService;
    
	@Autowired
	private ISaasProductPageService saasProductPageService;

	/**
	 * 查询产品页面树，不涉及权限过滤
	 */
	@ApiOperation(value = "查询产品页面树")
	@RequestMapping(value = "querySaasProductPageTree")
	public ResponseResult querySaasProductPageTree(@RequestBody @Validated SaasProductPageTreeVO treeVO) {
		// 验证产品信息是否存在
		SaasProduct saasProduct = this.saasProductService.getById(treeVO.getProductId());
		if (saasProduct == null) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "产品信息不存在");
		}
		// 获取该产品的下级页面权限信息，按照超管逻辑处理
		List<SaasProductPage> pageList = this.saasProductHandler.querySaasProductPageTree(treeVO.getAngle(), treeVO.getProductId(), null, null, true, treeVO.isShowPageAuth());
		saasProduct.setProductPages(pageList);
		return ResponseResultUtil.successResult(saasProduct);
	}

	/**
	 * 新增产品页面信息
	 */
	@ApiOperation(value = "保存")
	@RequestMapping(value = "createSaasProductPage")
	public ResponseResult createSaasProductPage(@RequestBody @Validated SaasProductPageCreateVO model) {
		// 验证产品信息是否存在
		SaasProduct saasProduct = this.saasProductService.getById(model.getProductId());
		if (saasProduct == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "产品信息不存在");
		}
		SaasProductPage saasProductPage = new SaasProductPage();
		BeanUtils.copyProperties(model, saasProductPage);
		saasProductPage.setId(null);
		saasProductPage.setUpdateUser(model.getAccountId());
		
		boolean result = this.saasProductPageService.save(saasProductPage);
		return result ? ResponseResultUtil.successResult("保存成功", saasProductPage.getId()) : ResponseResultUtil.errorResult("保存失败");
	}

	/**
	 * 更新产品页面信息
	 */
	@ApiOperation(value = "更新")
	@RequestMapping(value = "updateSaasProductPage")
	public ResponseResult updateSaasProductPage(@RequestBody @Validated SaasProductPageCreateVO model) {
		if (StringUtil.isBlank(model.getId())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0400.getCode(), "页面ID，不可为空");
		}
		// 验证产品信息是否存在
		SaasProduct saasProduct = this.saasProductService.getById(model.getProductId());
		if (saasProduct == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "产品信息不存在");
		}
		SaasProductPage saasProductPage = new SaasProductPage();
		BeanUtils.copyProperties(model, saasProductPage);
		saasProductPage.setUpdateUser(model.getAccountId());
		
		boolean result = this.saasProductPageService.updateById(saasProductPage);
		return result ? ResponseResultUtil.successResult("更新成功") : ResponseResultUtil.errorResult("更新失败");
	}

	/**
	 * 物理删除，落地页不允许删除，请优先变更为非落地页，级联删除此页面的权限点信息
	 */
	@ApiOperation(value = "删除")
	@RequestMapping(value = "deleteSaasProductPage")
	public ResponseResult deleteSaasProductPage(@RequestBody @Validated SaasProductPageDeleteVO model) {
		// 判断是否为落地页
		SaasProductPage productPage = this.saasProductPageService.getById(model.getId());
		if (productPage == null) {
			return ResponseResultUtil.errorResult(ResponseCode.C0320.getCode(), "产品页面信息不存在");
		}
		if (!SaasCommonConstant.STR_STATUS_0.equals(productPage.getPageFloor())) {
			return ResponseResultUtil.errorResult(ResponseCode.A0402.getCode(), "落地页不允许删除，请优先变更落地页所属");
		}
		// 删除，级联删除权限及其关联关系
		boolean result = this.saasProductHandler.deleteSaasProductPage(model.getId());
		return result ? ResponseResultUtil.successResult("删除成功") : ResponseResultUtil.errorResult("删除失败");
	}

}
